Attribute VB_Name = "mWinTrust"
'''''''''''''''''''''''''''''
'
' mWinTrust
' Ȩ 2009-2017 ZX Inc.
' 汾 1.0.3
' :  Windows 9x
'
' : ֤ǩ
'
'''''''''''''''''''''''''''''
Option Explicit

Private Declare Sub RtlFillMemory Lib "kernel32.dll" (ByVal Destination As Long, ByVal Length As Long, ByVal Fill As Byte)
Private Declare Function WinVerifyTrust Lib "wintrust.dll" (ByVal hWnd As Long, ByVal pgActionID As Long, ByVal pWVTData As Long) As Long
Private Declare Function GetLastError Lib "kernel32.dll" () As Long

Public Const ERROR_SUCCESS = 0&
Public Enum TRUST_E
    TRUST_SUCCESS = 0&
    TRUST_E_NO_SIGNATURE = &H800B0100
    TRUST_E_EXPIRED = &H800B0101
    TRUST_E_VALIDITY_PERIOD_NESTING = &H800B0102
    TRUST_E_CRITICAL = &H800B0105
    TRUST_E_PURPOSE = &H800B0106
    TRUST_E_ISSUER_CHAINING = &H800B0107
    TRUST_E_UNTRUSTED_ROOT = &H800B0109
    TRUST_E_CHAINING = &H800B010A
    TRUST_E_UNTRUSTED_TEST_ROOT = &H800B010D
    TRUST_E_CN_NO_MATCH = &H800B010F
    TRUST_E_WRONG_USAGE = &H800B0110
    TRUST_E_EXPLICIT_DISTRUST = &H800B0111
    TRUST_E_UNTRUSTED_CA = &H800B0112
    TRUST_E_SUBJECT_NOT_TRUSTED = &H800B0004
    CRYPT_E_SECURITY_SETTINGS = &H80092026
End Enum

Const WTD_UI_ALL = 1& 'Display all UI.
Const WTD_UI_NONE = 2& ' Display no UI.
Const WTD_UI_NOBAD = 3& ' Do not display any negative UI.
Const WTD_UI_NOGOOD = 4& ' Do not display any positive UI.

Const WTD_REVOKE_NONE = 0& 'No additional revocation checking will be done.
Const WTD_REVOKE_WHOLECHAIN = 1& ' Revocation checking will be done on the whole chain.
 
Const WTD_CHOICE_FILE = 1& ' Use the file pointed to by pFile.
Const WTD_CHOICE_CATALOG = 2& ' Use the catalog pointed to by pCatalog.
Const WTD_CHOICE_BLOB = 3& ' Use the BLOB pointed to by pBlob.
Const WTD_CHOICE_SIGNER = 4& ' Use the WINTRUST_SGNR_INFO structure pointed to by pSgnr.
Const WTD_CHOICE_CERT = 5& ' Use the certificate pointed to by pCert.

Const WTD_REVOCATION_CHECK_NONE = &H10&
Const WTD_SAFER_FLAG = &H100&
Const WTD_CACHE_ONLY_URL_RETRIEVAL = &H1000&

Private Type GUID
    D1 As Long
    D2 As Integer
    D3 As Integer
    D4(0 To 7) As Byte
End Type

Private Type WINTRUST_FILE_INFO
    cbStruct As Long
    pcwszFilePath As Long
    hFile As Long
    pgKnownSubject As Long
End Type

Private Type WINTRUST_DATA
    cbStruct As Long
    pPolicyCallbackData As Long
    pSIPClientData As Long
    dwUIChoice As Long
    fdwRevocationChecks As Long
    dwUnionChoice As Long
    'Union
    pUnionData As Long
    dwStateAction As Long
    hWVTStateData As Long
    pwszURLReference As Long
    dwProvFlags As Long
    dwUIContext As Long
End Type

Public Function VerifyEmbeddedSignature(pwszSourceFile As String) As Long
    Dim lStatus As Long
    Dim dwLastError As Long

    ' Initialize the WINTRUST_FILE_INFO structure.

    Dim FileData As WINTRUST_FILE_INFO
    
    RtlFillMemory VarPtr(FileData), LenB(FileData), 0
    
    FileData.cbStruct = LenB(FileData)
    FileData.pcwszFilePath = StrPtr(pwszSourceFile)
    FileData.hFile = 0
    FileData.pgKnownSubject = 0

    Dim WVTPolicyGUID As GUID
    
    '=WINTRUST_ACTION_GENERIC_VERIFY_V2
    WVTPolicyGUID.D1 = &HAAC56B
    WVTPolicyGUID.D2 = &HCD44
    WVTPolicyGUID.D3 = &H11D0
    WVTPolicyGUID.D4(0) = &H8C
    WVTPolicyGUID.D4(1) = &HC2
    WVTPolicyGUID.D4(2) = &H0
    WVTPolicyGUID.D4(3) = &HC0
    WVTPolicyGUID.D4(4) = &H4F
    WVTPolicyGUID.D4(5) = &HC2
    WVTPolicyGUID.D4(6) = &H95
    WVTPolicyGUID.D4(7) = &HEE
    
    Dim WinTrustData As WINTRUST_DATA

    '// Initialize the WinVerifyTrust input data structure.

    '// Default all fields to 0.
    RtlFillMemory VarPtr(WinTrustData), LenB(WinTrustData), 0
    WinTrustData.cbStruct = LenB(WinTrustData)
    '// Use default code signing EKU.
    WinTrustData.pPolicyCallbackData = 0
    '// No data to pass to SIP.
    WinTrustData.pSIPClientData = 0
    '// Disable WVT UI.
    WinTrustData.dwUIChoice = WTD_UI_NONE
    '// No revocation checking.
    WinTrustData.fdwRevocationChecks = WTD_REVOKE_NONE
    '// Verify an embedded signature on a file.
    WinTrustData.dwUnionChoice = WTD_CHOICE_FILE
    '// Default verification.
    WinTrustData.dwStateAction = 0
    '// Not applicable for default verification of embedded signature.
    WinTrustData.hWVTStateData = 0
    '// Not used.
    WinTrustData.pwszURLReference = 0
    '// Default.
    WinTrustData.dwProvFlags = WTD_REVOCATION_CHECK_NONE 'WTD_SAFER_FLAG Or WTD_CACHE_ONLY_URL_RETRIEVAL

'    // This is not applicable if there is no UI because it changes
'    // the UI to accommodate running applications instead of
'    // installing applications.
    WinTrustData.dwUIContext = 0

    '// Set pFile.
    WinTrustData.pUnionData = VarPtr(FileData)

    '// WinVerifyTrust verifies signatures as specified by the GUID
    '// and Wintrust_Data.
    lStatus = WinVerifyTrust(0, VarPtr(WVTPolicyGUID), VarPtr(WinTrustData))

'    Select Case (lStatus)
'        Case ERROR_SUCCESS
            
'            MsgBox "The file """ & pwszSourceFile & """ is signed and the signature was verified."
        
        
'        Case TRUST_E_NOSIGNATURE

'            dwLastError = GetLastError()
'            If (TRUST_E_NOSIGNATURE = dwLastError) Or (TRUST_E_SUBJECT_FORM_UNKNOWN = dwLastError) Or (TRUST_E_PROVIDER_UNKNOWN = dwLastError) Then

                '// The file was not signed.
'                MsgBox "The file """ & pwszSourceFile & """ is not signed."

'            Else

'                // The signature was not valid or there was an error
'                // opening the file.
'                MsgBox "An unknown error occurred trying to verify the signature of the """ & pwszSourceFile & """ file."
                    
'            End If

'        Case TRUST_E_EXPLICIT_DISTRUST
'            // The hash that represents the subject or the publisher
'            // is not allowed by the admin or user.
'            MsgBox "The signature is present, but specifically disallowed."
            

'        Case TRUST_E_SUBJECT_NOT_TRUSTED
            '// The user clicked "No" when asked to install and run.
'            MsgBox "The signature is present, but not trusted."


'        Case CRYPT_E_SECURITY_SETTINGS

'            MsgBox "CRYPT_E_SECURITY_SETTINGS - The hash " & _
                 "representing the subject or the publisher wasn't " & _
                 "explicitly trusted by the admin and admin policy " & _
                 "has disabled user trust. No signature, publisher " & _
                "or timestamp errors."


'        Case Else
            
'            // The UI was disabled in dwUIChoice or the admin policy
'            // has disabled user trust. lStatus contains the
'            // publisher or time stamp chain error.
'            MsgBox "Error is: 0x" & Hex$(lStatus) & "."
'    End Select

    VerifyEmbeddedSignature = lStatus


End Function

